--[[
HolmerAndRopaPotatoReady

Specialization to allow harvest potato with the Holmer and Ropa vehicles

Author: 	Ifko[nator]
Date: 		15.05.2020
Version:	3.0

History:	v1.0 @15.03.2016 - initial implemation in FS 15
			v2.0 @03.11.2016 - convert to FS 17
			v3.0 @15.05.2020 - convert to FS 19
]]

HolmerAndRopaPotatoReady = {};

function HolmerAndRopaPotatoReady.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(FillUnit, specializations) or SpecializationUtil.hasSpecialization(FruitPreparer, specializations);
end;

function HolmerAndRopaPotatoReady.registerEventListeners(vehicleType)
	local functionNames = {
		"onLoad",	
		"onUpdateTick"
	};

	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, HolmerAndRopaPotatoReady);
	end;
end;

function HolmerAndRopaPotatoReady:onLoad(savegame)
	local specFillUnit = self.spec_fillUnit;
	local specCutter = self.spec_cutter;

	if specCutter ~= nil then
		if specCutter.fruitTypes[FruitType.SUGARBEET] and not specCutter.fruitTypes[FruitType.POTATO] then
			specCutter.fruitTypes[FruitType.POTATO] = true;
		end;
	end;

	if specFillUnit ~= nil then
		local fillUnitConfigurationId = Utils.getNoNil(self.configurations["fillUnit"], 1);
		local baseKey = string.format("vehicle.fillUnit.fillUnitConfigurations.fillUnitConfiguration(%d).fillUnits", fillUnitConfigurationId - 1);
		
		local fillUnitNumber = 0;

    	while true do
			local key = string.format("%s.fillUnit(%d)", baseKey, fillUnitNumber);

			if not hasXMLProperty(self.xmlFile, key) then
    	        break;
			end;

			local fillTypes = getXMLString(self.xmlFile, key .. "#fillTypes");
		
			if fillTypes ~= nil then
				if specFillUnit.fillUnits[fillUnitNumber + 1].supportedFillTypes[FillType.SUGARBEET] and not specFillUnit.fillUnits[fillUnitNumber + 1].supportedFillTypes[FillType.POTATO] then
					specFillUnit.fillUnits[fillUnitNumber + 1].supportedFillTypes[FillType.POTATO] = true;
				end;
			end;

			fillUnitNumber = fillUnitNumber + 1;
		end;
	end;
end;

function HolmerAndRopaPotatoReady:onUpdateTick(dt)
	local specFruitPreparer = self.spec_fruitPreparer;
	
	if specFruitPreparer == nil or self.getCombine == nil then
		return;
	else
		local combine = self:getCombine();
		
		if combine ~= nil and combine:getIsTurnedOn() then		
			local specCutter = self.spec_cutter;
			local specFillUnit = combine.spec_fillUnit;
			local specCombine = combine.spec_combine;

			if specFillUnit.fillUnits ~= nil then	
				for _, fillUnit in pairs(specFillUnit.fillUnits) do
					if fillUnit.fillType == FillType.UNKNOWN then
						if specFruitPreparer.fruitType == FruitType.POTATO then
							local sugarBeet = g_fruitTypeManager:getFruitTypeByName("sugarBeet");
							
							if sugarBeet ~= nil then
								specFruitPreparer.fruitType = sugarBeet.index;
								
								if self.setAIFruitRequirements ~= nil then
               						self:setAIFruitRequirements(sugarBeet.index, sugarBeet.minPreparingGrowthState, sugarBeet.maxPreparingGrowthState);
									   
									local aiUsePreparedState = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.fruitPreparer#aiUsePreparedState"), specCutter ~= nil);
									   
									if aiUsePreparedState then
               						    self:addAIFruitRequirement(sugarBeet.index, sugarBeet.preparedGrowthState, sugarBeet.preparedGrowthState);
               						end;
								end;

								combine.fruitPreparerFruitType = sugarBeet.index;
							end;
						elseif specFruitPreparer.fruitType == FruitType.SUGARBEET then
							local potato = g_fruitTypeManager:getFruitTypeByName("potato");
				
							if potato ~= nil then
								specFruitPreparer.fruitType = potato.index;
								
								if self.setAIFruitRequirements ~= nil then
               						self:setAIFruitRequirements(potato.index, potato.minPreparingGrowthState, potato.maxPreparingGrowthState);
									   
									local aiUsePreparedState = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.fruitPreparer#aiUsePreparedState"), specCutter ~= nil);
									   
									if aiUsePreparedState then
               						    self:addAIFruitRequirement(potato.index, potato.preparedGrowthState, potato.preparedGrowthState);
               						end;
								end;

								combine.fruitPreparerFruitType = potato.index;
							end;
						end;
					elseif specFruitPreparer.currentInputFruitType ~= specFruitPreparer.fruitType then	
						local currentFruitType = specFruitPreparer.currentInputFruitType;
						
						if combine ~= nil then	
							currentFruitType = g_fruitTypeManager:getFruitTypeByName(g_fillTypeManager:getFillTypeByIndex(fillUnit.fillType).name);
						end;
						
						if currentFruitType ~= nil then
							specFruitPreparer.fruitType = currentFruitType.index;
							
							if self.setAIFruitRequirements ~= nil then
               					self:setAIFruitRequirements(currentFruitType.index, currentFruitType.minPreparingGrowthState, currentFruitType.maxPreparingGrowthState);
								   
								local aiUsePreparedState = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.fruitPreparer#aiUsePreparedState"), specCutter ~= nil);
								   
								if aiUsePreparedState then
               					    self:addAIFruitRequirement(currentFruitType.index, currentFruitType.preparedGrowthState, currentFruitType.preparedGrowthState);
               					end;
							end;
							
							combine.fruitPreparerFruitType = currentFruitType.index;

							specFruitPreparer.currentInputFruitType = specFruitPreparer.fruitType;
						end;
					end;
				end;
			end;
		end;
	end;
end;